module Options : sig
type page = {
  default_uri_scheme : string;
  new_page_default_uri : string;
}
type navigation = {
  page : page;
  tabs_position : Gtk.Tags.position;
  new_tab_placement : Zipper.where_add;
  post_close_activate : Zipper.where_remove;
}

type browser = {
  navigation : navigation;
}

val default : unit -> browser
end
module OptionsDescr : sig
val browser : Lib.Options.category Lib.Options.wrap
end
module Browser : sig
module Page0 :
  sig
    type options = Options.page
    type t = {
      uri : string;
      title : string;
      history : string list;
      options : options;
      id : LablgtkReact.id;
      parent_id : LablgtkReact.id option;
      request : [ `Close | `Popup | `None ];
    }
    type init = LablgtkReact.id * LablgtkReact.id option * options
    type action_common = [
      | `Uri of string
      | `Title of string
      | `SetOptions of options
      | `Close
    ]
    type action_functional = [ action_common | `RequestClose | `Popup | `PopupDone ]
    type action_ui = [ action_common | `None ]
    val create : init -> t
    val on_action_func : t -> action_functional -> t * action_ui
  end
module Navigation0 :
  sig
    type t = { z : Page0.t Zipper.t; options : Options.navigation; }
    type init = Options.navigation
    type action_common = [
      | `Nth of int
      | `MoveTo of int
      | `SetOptions of Options.navigation
    ]
    type action_functional = [
      action_common
      | `CloseCurrent
      | `Set of Page0.t
    ]
    type action_ui = [
      action_common
      | `Close of (LablgtkReact.id * int option)
      | `Popup of LablgtkReact.id
      | `None
    ]
    val create : Options.navigation -> t
    val on_action_func : t -> action_functional -> t * action_ui
  end
module Browser0 :
  sig
    type options = Options.browser
    type t = { options : options; navigation : Navigation0.t; }
    type init = options
    type action_common = [ `SetOptions of options ]
    type action_functional = [ action_common | `Set of Navigation0.t ]
    type action_ui = [ action_common | `None ]
    val create : options -> t
    val on_action_func : t -> action_functional -> t * action_ui
  end
end
module Types : sig
  module Page :
    sig
      class core :
        ?state_callbacks:(Browser.Page0.t -> unit) list ->
        ?msg_callbacks:(Browser.Page0.action_ui -> unit) list ->
        Browser.Page0.init ->
        object
          method msg_callbacks : Browser.Page0.action_ui LablgtkReact.callbacks
          method send : Browser.Page0.action_functional -> unit
          method state_callbacks : Browser.Page0.t LablgtkReact.callbacks
        end
    end
  module Navigation :
    sig
      class core :
        ?state_callbacks:(Browser.Navigation0.t -> unit) list ->
        ?msg_callbacks:(Browser.Navigation0.action_ui -> unit) list ->
        Browser.Navigation0.init ->
        object
          method msg_callbacks : Browser.Navigation0.action_ui LablgtkReact.callbacks
          method send : Browser.Navigation0.action_functional -> unit
          method state_callbacks : Browser.Navigation0.t LablgtkReact.callbacks
        end
    end
  module Browser :
    sig
      class core :
        ?state_callbacks:(Browser.Browser0.t -> unit) list ->
        ?msg_callbacks:(Browser.Browser0.action_ui -> unit) list ->
        Browser.Browser0.init ->
        object
          method msg_callbacks : Browser.Browser0.action_ui LablgtkReact.callbacks
          method send : Browser.Browser0.action_functional -> unit
          method state_callbacks : Browser.Browser0.t LablgtkReact.callbacks
        end
    end
end
